@@ -106,7 +106,7 @@ static void at_csca_set(struct ofono_sms *sms,
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	char buf[64];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	snprintf(buf, sizeof(buf), "AT+CSCA=\"%s\",%d", sca->number, sca->type);
@@ -116,8 +116,7 @@ static void at_csca_set(struct ofono_sms *sms,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, user_data);
 }
@@ -174,7 +173,7 @@ static void at_csca_query(struct ofono_sms *sms, ofono_sms_sca_query_cb_t cb,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(data->chat, "AT+CSCA?", csca_prefix,
@@ -182,8 +181,7 @@ static void at_csca_query(struct ofono_sms *sms, ofono_sms_sca_query_cb_t cb,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, NULL, user_data);
 }
@@ -229,7 +227,7 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 	char buf[512];
 	int len;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (mms) {
@@ -246,8 +244,7 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, user_data);
 }
@@ -270,7 +267,7 @@ static void at_cgsms_set(struct ofono_sms *sms, int bearer,
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	char buf[64];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	snprintf(buf, sizeof(buf), "AT+CGSMS=%d", bearer);
@@ -280,8 +277,7 @@ static void at_cgsms_set(struct ofono_sms *sms, int bearer,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, user_data);
 }
@@ -323,7 +319,7 @@ static void at_cgsms_query(struct ofono_sms *sms,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(data->chat, "AT+CGSMS?", cgsms_prefix,
@@ -331,8 +327,7 @@ static void at_cgsms_query(struct ofono_sms *sms,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, user_data);
 }
@@ -370,6 +365,8 @@ static inline void at_ack_delivery(struct ofono_sms *sms)
 	struct sms_data *data = ofono_sms_get_data(sms);
 	char buf[256];
 
+	DBG("");
+
 	/* We must acknowledge the PDU using CNMA */
 	if (data->cnma_ack_pdu)
 		snprintf(buf, sizeof(buf), "AT+CNMA=1,%d\r%s",
@@ -445,6 +442,8 @@ static void at_cmgr_notify(GAtResult *result, gpointer user_data)
 	long pdu_len;
 	int tpdu_len;
 
+	DBG("");
+
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CMGR:"))
@@ -593,6 +592,8 @@ static void at_cmgl_done(struct ofono_sms *sms)
 {
 	struct sms_data *data = ofono_sms_get_data(sms);
 
+	DBG("");
+
 	if (data->incoming == AT_UTIL_SMS_STORE_MT &&
 			data->store == AT_UTIL_SMS_STORE_ME) {
 		at_cmgl_set_cpms(sms, AT_UTIL_SMS_STORE_SM);
@@ -626,6 +627,8 @@ static void at_cmgl_notify(GAtResult *result, gpointer user_data)
 	int status;
 	char buf[16];
 
+	DBG("");
+
 	g_at_result_iter_init(&iter, result);
 
 	while (g_at_result_iter_next(&iter, "+CMGL:")) {
@@ -793,15 +796,22 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 	const char *mode;
 	int len = sprintf(buf, "AT+CNMI=");
 
-	if (data->vendor == OFONO_VENDOR_QUALCOMM_MSM ||
-			data->vendor == OFONO_VENDOR_HUAWEI ||
-			data->vendor == OFONO_VENDOR_NOVATEL)
+	DBG("");
+
+	switch (data->vendor) {
+	case OFONO_VENDOR_GOBI:
+	case OFONO_VENDOR_QUALCOMM_MSM:
+	case OFONO_VENDOR_NOVATEL:
+	case OFONO_VENDOR_HUAWEI:
 		/* MSM devices advertise support for mode 2, but return an
 		 * error if we attempt to actually use it. */
 		mode = "1";
-	else
+		break;
+	default:
 		/* Sounds like 2 is the sanest mode */
 		mode = "2310";
+		break;
+	}
 
 	if (!append_cnmi_element(buf, &len, cnmi_opts[0], mode, FALSE))
 		return FALSE;
@@ -821,10 +831,14 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 	 * ack it with error "CNMA not expected."  However, not acking it
 	 * sends the device into la-la land.
 	 */
-	if (data->vendor == OFONO_VENDOR_NOVATEL)
+	switch (data->vendor) {
+	case OFONO_VENDOR_NOVATEL:
 		mode = "20";
-	else
+		break;
+	default:
 		mode = "120";
+		break;
+	}
 
 	/*
 	 * Try to deliver Status-Reports via +CDS, then CDSI or don't
@@ -847,6 +861,8 @@ static void construct_ack_pdu(struct sms_data *d)
 	int len;
 	int tpdu_len;
 
+	DBG("");
+
 	memset(&ackpdu, 0, sizeof(ackpdu));
 
 	ackpdu.type = SMS_TYPE_DELIVER_REPORT_ACK;
@@ -859,8 +875,7 @@ static void construct_ack_pdu(struct sms_data *d)
 		goto err;
 
 	d->cnma_ack_pdu = encode_hex(pdu, tpdu_len, 0);
-
-	if (!d->cnma_ack_pdu)
+	if (d->cnma_ack_pdu == NULL)
 		goto err;
 
 	d->cnma_ack_pdu_len = tpdu_len;
@@ -910,10 +925,15 @@ static void at_cnmi_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		supported = TRUE;
 
 	/* support for ack pdu is not working */
-	if (data->vendor == OFONO_VENDOR_NOVATEL ||
-			data->vendor == OFONO_VENDOR_HUAWEI ||
-			data->vendor == OFONO_VENDOR_OPTION_HSO)
+	switch (data->vendor) {
+	case OFONO_VENDOR_IFX:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_NOVATEL:
+	case OFONO_VENDOR_OPTION_HSO:
 		goto out;
+	default:
+		break;
+	}
 
 	if (data->cnma_enabled)
 		construct_ack_pdu(data);
@@ -1131,13 +1151,16 @@ static void at_csms_status_cb(gboolean ok, GAtResult *result,
 			goto out;
 
 
-		if (data->vendor == OFONO_VENDOR_HUAWEI ||
-				data->vendor == OFONO_VENDOR_NOVATEL) {
+		switch (data->vendor) {
+		case OFONO_VENDOR_HUAWEI:
+		case OFONO_VENDOR_NOVATEL:
 			g_at_result_iter_skip_next(&iter);
 			service = 0;
-		} else {
+			break;
+		default:
 			if (!g_at_result_iter_next_number(&iter, &service))
 				goto out;
+			break;
 		}
 
 		if (!g_at_result_iter_next_number(&iter, &mt))
@@ -1212,12 +1235,12 @@ static int at_sms_probe(struct ofono_sms *sms, unsigned int vendor,
 	struct sms_data *data;
 
 	data = g_new0(struct sms_data, 1);
-	data->chat = chat;
+	data->chat = g_at_chat_clone(chat);
 	data->vendor = vendor;
 
 	ofono_sms_set_data(sms, data);
 
-	g_at_chat_send(chat, "AT+CSMS=?", csms_prefix,
+	g_at_chat_send(data->chat, "AT+CSMS=?", csms_prefix,
 			at_csms_query_cb, sms, NULL);
 
 	return 0;
@@ -1227,13 +1250,15 @@ static void at_sms_remove(struct ofono_sms *sms)
 {
 	struct sms_data *data = ofono_sms_get_data(sms);
 
-	if (data->cnma_ack_pdu)
-		g_free(data->cnma_ack_pdu);
+	g_free(data->cnma_ack_pdu);
 
 	if (data->timeout_source > 0)
 		g_source_remove(data->timeout_source);
 
+	g_at_chat_unref(data->chat);
 	g_free(data);
+
+	ofono_sms_set_data(sms, NULL);
 }
 
 static struct ofono_sms_driver driver = {
@@ -1247,12 +1272,12 @@ static struct ofono_sms_driver driver = {
 	.bearer_set	= at_cgsms_set,
 };
 
-void at_sms_init()
+void at_sms_init(void)
 {
 	ofono_sms_driver_register(&driver);
 }
 
-void at_sms_exit()
+void at_sms_exit(void)
 {
 	ofono_sms_driver_unregister(&driver);
 }
