@@ -180,8 +180,8 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 static void ste_dial(struct ofono_voicecall *vc,
 			const struct ofono_phone_number *ph,
-			enum ofono_clir_option clir, enum ofono_cug_option cug,
-			ofono_voicecall_cb_t cb, void *data)
+			enum ofono_clir_option clir, ofono_voicecall_cb_t cb,
+			void *data)
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct cb_data *cbd = cb_data_new(cb, data);
@@ -208,14 +208,6 @@ static void ste_dial(struct ofono_voicecall *vc,
 		break;
 	}
 
-	switch (cug) {
-	case OFONO_CUG_OPTION_DEFAULT:
-		break;
-	case OFONO_CUG_OPTION_INVOCATION:
-		strcat(buf, "G");
-		break;
-	}
-
 	strcat(buf, ";");
 
 	if (g_at_chat_send(vd->chat, buf, none_prefix,
