@@ -736,6 +736,8 @@ static void lockdown_remove(struct ofono_modem *modem)
 	}
 
 	g_free(modem->lock_owner);
+	modem->lock_owner = NULL;
+
 	modem->lockdown = FALSE;
 }
 
@@ -794,6 +796,7 @@ static DBusMessage *set_property_lockdown(struct ofono_modem *modem,
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	ofono_bool_t lockdown;
+	dbus_bool_t powered;
 	const char *caller;
 	int err;
 
@@ -805,54 +808,55 @@ static DBusMessage *set_property_lockdown(struct ofono_modem *modem,
 	if (modem->pending != NULL)
 		return __ofono_error_busy(msg);
 
+	caller = dbus_message_get_sender(msg);
+
+	if (modem->lockdown && g_strcmp0(caller, modem->lock_owner))
+		return __ofono_error_access_denied(msg);
+
 	if (modem->lockdown == lockdown)
 		return dbus_message_new_method_return(msg);
 
-	caller = dbus_message_get_sender(msg);
+	if (lockdown == FALSE) {
+		lockdown_remove(modem);
+		goto done;
+	}
 
-	if (lockdown) {
-		dbus_bool_t powered;
+	modem->lock_owner = g_strdup(caller);
 
-		modem->lock_owner = g_strdup(caller);
-
-		modem->lock_watch = g_dbus_add_disconnect_watch(conn,
+	modem->lock_watch = g_dbus_add_disconnect_watch(conn,
 				modem->lock_owner, lockdown_disconnect,
 				modem, NULL);
 
-		if (modem->lock_watch == 0) {
-			g_free(modem->lock_owner);
+	if (modem->lock_watch == 0) {
+		g_free(modem->lock_owner);
+		modem->lock_owner = NULL;
+
+		return __ofono_error_failed(msg);
+	}
+
+	modem->lockdown = lockdown;
+
+	if (modem->powered == FALSE)
+		goto done;
+
+	err = set_powered(modem, FALSE);
+	if (err < 0) {
+		if (err != -EINPROGRESS) {
+			lockdown_remove(modem);
 			return __ofono_error_failed(msg);
 		}
 
-		modem->lockdown = lockdown;
-
-		if (!modem->powered)
-			goto done;
-
-		err = set_powered(modem, FALSE);
-		if (err < 0) {
-			if (err != -EINPROGRESS) {
-				lockdown_remove(modem);
-				return __ofono_error_failed(msg);
-			}
-
-			modem->pending = dbus_message_ref(msg);
-			modem->timeout = g_timeout_add_seconds(20,
+		modem->pending = dbus_message_ref(msg);
+		modem->timeout = g_timeout_add_seconds(20,
 						set_powered_timeout, modem);
-			return NULL;
-		}
+		return NULL;
+	}
 
-		powered = FALSE;
-		ofono_dbus_signal_property_changed(conn, modem->path,
+	powered = FALSE;
+	ofono_dbus_signal_property_changed(conn, modem->path,
 					OFONO_MODEM_INTERFACE,
 					"Powered", DBUS_TYPE_BOOLEAN,
 					&powered);
-	} else {
-		if (g_strcmp0(caller, modem->lock_owner))
-			return __ofono_error_access_denied(msg);
-
-		lockdown_remove(modem);
-	}
 
 done:
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
@@ -1075,6 +1079,7 @@ static const struct {
 	{ OFONO_MESSAGE_MANAGER_INTERFACE,		"sms"	},
 	{ OFONO_SIM_MANAGER_INTERFACE,			"sim"	},
 	{ OFONO_STK_INTERFACE,				"stk"	},
+	{ OFONO_SUPPLEMENTARY_SERVICES_INTERFACE,	"ussd"	},
 	{ OFONO_CONNECTION_MANAGER_INTERFACE,		"gprs"	},
 	{ OFONO_TEXT_TELEPHONY_INTERFACE,		"tty"	},
 	{ OFONO_LOCATION_REPORTING_INTERFACE,		"gps"	},
